# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.analytics import analytics_service_pb2 as qwak_dot_analytics_dot_analytics__service__pb2


class AnalyticsQueryServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Query = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/Query',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.QueryRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.QueryResponse.FromString,
                )
        self.ExecuteDdlQuery = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/ExecuteDdlQuery',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.ExecuteDdlQueryRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.ExecuteDdlQueryResponse.FromString,
                )
        self.GetQueryResultsPage = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/GetQueryResultsPage',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultsPageRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultsPageResponse.FromString,
                )
        self.GetQueryStatus = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/GetQueryStatus',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryStatusRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryStatusResponse.FromString,
                )
        self.DescribeTable = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/DescribeTable',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.DescribeTableRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.DescribeTableResponse.FromString,
                )
        self.ListTables = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/ListTables',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.ListTableRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.ListTableResponse.FromString,
                )
        self.CreateTable = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/CreateTable',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateTableRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateTableResponse.FromString,
                )
        self.GetQueryResultDownloadURL = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/GetQueryResultDownloadURL',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultDownloadURLRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultDownloadURLResponse.FromString,
                )
        self.OptimizeTable = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/OptimizeTable',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.OptimizeTableRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.OptimizeTableResponse.FromString,
                )
        self.CreateDatabase = channel.unary_unary(
                '/qwak.analytics.AnalyticsQueryService/CreateDatabase',
                request_serializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateDatabaseRequest.SerializeToString,
                response_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateDatabaseResponse.FromString,
                )


class AnalyticsQueryServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Query(self, request, context):
        """Initiates a query request from the user. Returns a query ID which the client should start subscribing on
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExecuteDdlQuery(self, request, context):
        """Executing ddl query and waiting for the execution to finish
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetQueryResultsPage(self, request, context):
        """Fetch the query results page of a submitted query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetQueryStatus(self, request, context):
        """Get Query status of a submitted query
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeTable(self, request, context):
        """Describe table - shows its list of column names and types
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListTables(self, request, context):
        """List all table names in the system
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateTable(self, request, context):
        """Creates table
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetQueryResultDownloadURL(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def OptimizeTable(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateDatabase(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AnalyticsQueryServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Query': grpc.unary_unary_rpc_method_handler(
                    servicer.Query,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.QueryRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.QueryResponse.SerializeToString,
            ),
            'ExecuteDdlQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.ExecuteDdlQuery,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.ExecuteDdlQueryRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.ExecuteDdlQueryResponse.SerializeToString,
            ),
            'GetQueryResultsPage': grpc.unary_unary_rpc_method_handler(
                    servicer.GetQueryResultsPage,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultsPageRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultsPageResponse.SerializeToString,
            ),
            'GetQueryStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetQueryStatus,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryStatusRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryStatusResponse.SerializeToString,
            ),
            'DescribeTable': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeTable,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.DescribeTableRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.DescribeTableResponse.SerializeToString,
            ),
            'ListTables': grpc.unary_unary_rpc_method_handler(
                    servicer.ListTables,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.ListTableRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.ListTableResponse.SerializeToString,
            ),
            'CreateTable': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateTable,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateTableRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateTableResponse.SerializeToString,
            ),
            'GetQueryResultDownloadURL': grpc.unary_unary_rpc_method_handler(
                    servicer.GetQueryResultDownloadURL,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultDownloadURLRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultDownloadURLResponse.SerializeToString,
            ),
            'OptimizeTable': grpc.unary_unary_rpc_method_handler(
                    servicer.OptimizeTable,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.OptimizeTableRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.OptimizeTableResponse.SerializeToString,
            ),
            'CreateDatabase': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateDatabase,
                    request_deserializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateDatabaseRequest.FromString,
                    response_serializer=qwak_dot_analytics_dot_analytics__service__pb2.CreateDatabaseResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.analytics.AnalyticsQueryService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AnalyticsQueryService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Query(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/Query',
            qwak_dot_analytics_dot_analytics__service__pb2.QueryRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.QueryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExecuteDdlQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/ExecuteDdlQuery',
            qwak_dot_analytics_dot_analytics__service__pb2.ExecuteDdlQueryRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.ExecuteDdlQueryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetQueryResultsPage(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/GetQueryResultsPage',
            qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultsPageRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultsPageResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetQueryStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/GetQueryStatus',
            qwak_dot_analytics_dot_analytics__service__pb2.GetQueryStatusRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.GetQueryStatusResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DescribeTable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/DescribeTable',
            qwak_dot_analytics_dot_analytics__service__pb2.DescribeTableRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.DescribeTableResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListTables(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/ListTables',
            qwak_dot_analytics_dot_analytics__service__pb2.ListTableRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.ListTableResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateTable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/CreateTable',
            qwak_dot_analytics_dot_analytics__service__pb2.CreateTableRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.CreateTableResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetQueryResultDownloadURL(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/GetQueryResultDownloadURL',
            qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultDownloadURLRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.GetQueryResultDownloadURLResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def OptimizeTable(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/OptimizeTable',
            qwak_dot_analytics_dot_analytics__service__pb2.OptimizeTableRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.OptimizeTableResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CreateDatabase(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.analytics.AnalyticsQueryService/CreateDatabase',
            qwak_dot_analytics_dot_analytics__service__pb2.CreateDatabaseRequest.SerializeToString,
            qwak_dot_analytics_dot_analytics__service__pb2.CreateDatabaseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/execution/v1/jobs/job.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from _qwak_proto.qwak.execution.v1 import backfill_pb2 as qwak_dot_execution_dot_v1_dot_backfill__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n qwak/execution/v1/jobs/job.proto\x12$qwak.feature.store.execution.v1.jobs\x1a\x1fgoogle/protobuf/timestamp.proto\x1a qwak/execution/v1/backfill.proto\"\xc0\x05\n\tJobRecord\x12\x0e\n\x06job_id\x18\x01 \x01(\t\x12\x17\n\rlegacy_run_id\x18\x02 \x01(\x03H\x00\x12\x16\n\x0c\x65xecution_id\x18\x03 \x01(\tH\x00\x12\x17\n\x0f\x66\x65\x61tureset_name\x18\x04 \x01(\t\x12\x15\n\rfeatureset_id\x18\x05 \x01(\t\x12\x16\n\x0e\x65nvironment_id\x18\x06 \x01(\t\x12\x12\n\naccount_id\x18\x07 \x01(\t\x12\x43\n\njob_status\x18\x08 \x01(\x0e\x32/.qwak.feature.store.execution.v1.jobs.JobStatus\x12\x31\n\rcreation_time\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nstart_time\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12G\n\x0cjob_metadata\x18\x0c \x01(\x0b\x32\x31.qwak.feature.store.execution.v1.jobs.JobMetadata\x12R\n\njob_labels\x18\r \x03(\x0b\x32>.qwak.feature.store.execution.v1.jobs.JobRecord.JobLabelsEntry\x12\x0c\n\x04\x63ode\x18\x0e \x01(\t\x12H\n\x04info\x18\x0f \x01(\x0b\x32\x38.qwak.feature.store.execution.v1.jobs.FailureInformationH\x01\x1a\x30\n\x0eJobLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\t\n\x07\x65xec_idB\x0e\n\x0c\x66\x61ilure_info\"\xea\x01\n\x0bJobMetadata\x12`\n\x18\x62\x61tch_ingestion_metadata\x18\x01 \x01(\x0b\x32<.qwak.feature.store.execution.v1.jobs.BatchIngestionMetadataH\x00\x12h\n!batch_backfill_ingestion_metadata\x18\x02 \x01(\x0b\x32;.qwak.feature.store.execution.v1.jobs.BatchBackfillMetadataH\x00\x42\x0f\n\rmetadata_type\"\xe7\x01\n\x16\x42\x61tchIngestionMetadata\x12<\n\x18ingested_data_start_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16ingested_data_end_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x19\n\x11num_affected_rows\x18\x03 \x01(\x03\x12\x38\n\x14\x62\x61tch_execution_date\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xd6\x03\n\x15\x42\x61tchBackfillMetadata\x12\x19\n\x11num_interval_rows\x18\x01 \x01(\x03\x12\x44\n\rbackfill_spec\x18\x02 \x01(\x0b\x32-.qwak.feature.store.execution.v1.BackfillSpec\x12\x19\n\x11num_affected_rows\x18\x03 \x01(\x03\x12\x41\n\x1doffline_processing_start_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12?\n\x1boffline_processing_end_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12@\n\x1conline_processing_start_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12>\n\x1aonline_processing_end_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12;\n\x17last_batch_logical_date\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"b\n\x12\x46\x61ilureInformation\x12\x1e\n\x16original_error_message\x18\x01 \x01(\t\x12\x11\n\x07message\x18\x02 \x01(\tH\x00\x42\x19\n\x17\x66ormatted_error_message*\xac\x01\n\tJobStatus\x12\x16\n\x12JOB_STATUS_INVALID\x10\x00\x12\x18\n\x14JOB_STATUS_COMPLETED\x10\x03\x12\x15\n\x11JOB_STATUS_FAILED\x10\x04\x12\x18\n\x14JOB_STATUS_TOO_EARLY\x10\x05\x12\x1d\n\x19LEGACY_JOB_STATUS_RUNNING\x10\x06\x12\x1d\n\x19LEGACY_JOB_STATUS_PENDING\x10\x07\x42G\n*com.qwak.ai.features.execution.api.v1.jobsP\x01Z\x17qwak/fsexecution;fsjobsb\x06proto3')

_JOBSTATUS = DESCRIPTOR.enum_types_by_name['JobStatus']
JobStatus = enum_type_wrapper.EnumTypeWrapper(_JOBSTATUS)
JOB_STATUS_INVALID = 0
JOB_STATUS_COMPLETED = 3
JOB_STATUS_FAILED = 4
JOB_STATUS_TOO_EARLY = 5
LEGACY_JOB_STATUS_RUNNING = 6
LEGACY_JOB_STATUS_PENDING = 7


_JOBRECORD = DESCRIPTOR.message_types_by_name['JobRecord']
_JOBRECORD_JOBLABELSENTRY = _JOBRECORD.nested_types_by_name['JobLabelsEntry']
_JOBMETADATA = DESCRIPTOR.message_types_by_name['JobMetadata']
_BATCHINGESTIONMETADATA = DESCRIPTOR.message_types_by_name['BatchIngestionMetadata']
_BATCHBACKFILLMETADATA = DESCRIPTOR.message_types_by_name['BatchBackfillMetadata']
_FAILUREINFORMATION = DESCRIPTOR.message_types_by_name['FailureInformation']
JobRecord = _reflection.GeneratedProtocolMessageType('JobRecord', (_message.Message,), {

  'JobLabelsEntry' : _reflection.GeneratedProtocolMessageType('JobLabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _JOBRECORD_JOBLABELSENTRY,
    '__module__' : 'qwak.execution.v1.jobs.job_pb2'
    # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.jobs.JobRecord.JobLabelsEntry)
    })
  ,
  'DESCRIPTOR' : _JOBRECORD,
  '__module__' : 'qwak.execution.v1.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.jobs.JobRecord)
  })
_sym_db.RegisterMessage(JobRecord)
_sym_db.RegisterMessage(JobRecord.JobLabelsEntry)

JobMetadata = _reflection.GeneratedProtocolMessageType('JobMetadata', (_message.Message,), {
  'DESCRIPTOR' : _JOBMETADATA,
  '__module__' : 'qwak.execution.v1.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.jobs.JobMetadata)
  })
_sym_db.RegisterMessage(JobMetadata)

BatchIngestionMetadata = _reflection.GeneratedProtocolMessageType('BatchIngestionMetadata', (_message.Message,), {
  'DESCRIPTOR' : _BATCHINGESTIONMETADATA,
  '__module__' : 'qwak.execution.v1.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.jobs.BatchIngestionMetadata)
  })
_sym_db.RegisterMessage(BatchIngestionMetadata)

BatchBackfillMetadata = _reflection.GeneratedProtocolMessageType('BatchBackfillMetadata', (_message.Message,), {
  'DESCRIPTOR' : _BATCHBACKFILLMETADATA,
  '__module__' : 'qwak.execution.v1.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.jobs.BatchBackfillMetadata)
  })
_sym_db.RegisterMessage(BatchBackfillMetadata)

FailureInformation = _reflection.GeneratedProtocolMessageType('FailureInformation', (_message.Message,), {
  'DESCRIPTOR' : _FAILUREINFORMATION,
  '__module__' : 'qwak.execution.v1.jobs.job_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.jobs.FailureInformation)
  })
_sym_db.RegisterMessage(FailureInformation)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n*com.qwak.ai.features.execution.api.v1.jobsP\001Z\027qwak/fsexecution;fsjobs'
  _JOBRECORD_JOBLABELSENTRY._options = None
  _JOBRECORD_JOBLABELSENTRY._serialized_options = b'8\001'
  _JOBSTATUS._serialized_start=1893
  _JOBSTATUS._serialized_end=2065
  _JOBRECORD._serialized_start=142
  _JOBRECORD._serialized_end=846
  _JOBRECORD_JOBLABELSENTRY._serialized_start=771
  _JOBRECORD_JOBLABELSENTRY._serialized_end=819
  _JOBMETADATA._serialized_start=849
  _JOBMETADATA._serialized_end=1083
  _BATCHINGESTIONMETADATA._serialized_start=1086
  _BATCHINGESTIONMETADATA._serialized_end=1317
  _BATCHBACKFILLMETADATA._serialized_start=1320
  _BATCHBACKFILLMETADATA._serialized_end=1790
  _FAILUREINFORMATION._serialized_start=1792
  _FAILUREINFORMATION._serialized_end=1890
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/instance_template/instance_template.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.user_application.common.v0 import resources_pb2 as qwak_dot_user__application_dot_common_dot_v0_dot_resources__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.qwak/instance_template/instance_template.proto\x12!qwak.management.instance_template\x1a/qwak/user_application/common/v0/resources.proto\"\xb0\x02\n\x14InstanceTemplateSpec\x12\n\n\x02id\x18\x01 \x01(\t\x12\x14\n\x0c\x64isplay_name\x18\x02 \x01(\t\x12\r\n\x05order\x18\x03 \x01(\x05\x12\x0f\n\x07\x65nabled\x18\x04 \x01(\x08\x12S\n\x15pod_compute_resources\x18\x05 \x01(\x0b\x32\x34.qwak.user_application.common.v0.PodComputeResources\x12\x46\n\rinstance_type\x18\x06 \x01(\x0e\x32/.qwak.management.instance_template.InstanceType\x12\x0b\n\x03qpu\x18\x07 \x01(\x02\x12\x15\n\raws_supported\x18\x08 \x01(\x08\x12\x15\n\rgcp_supported\x18\t \x01(\x08\"e\n\x0eInstanceFilter\x12S\n\x14instance_type_filter\x18\x01 \x01(\x0e\x32\x35.qwak.management.instance_template.InstanceTypeFilter*\x94\x01\n\x12InstanceTypeFilter\x12$\n INSTANCE_TYPE_FILTER_UNSPECIFIED\x10\x00\x12\x1c\n\x18INSTANCE_TYPE_FILTER_ALL\x10\x01\x12\x1c\n\x18INSTANCE_TYPE_FILTER_CPU\x10\x02\x12\x1c\n\x18INSTANCE_TYPE_FILTER_GPU\x10\x03*[\n\x0cInstanceType\x12\x1d\n\x19INSTANCE_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11INSTANCE_TYPE_CPU\x10\x01\x12\x15\n\x11INSTANCE_TYPE_GPU\x10\x02\x42\x30\n,com.qwak.ai.management.instance_template.apiP\x01\x62\x06proto3')

_INSTANCETYPEFILTER = DESCRIPTOR.enum_types_by_name['InstanceTypeFilter']
InstanceTypeFilter = enum_type_wrapper.EnumTypeWrapper(_INSTANCETYPEFILTER)
_INSTANCETYPE = DESCRIPTOR.enum_types_by_name['InstanceType']
InstanceType = enum_type_wrapper.EnumTypeWrapper(_INSTANCETYPE)
INSTANCE_TYPE_FILTER_UNSPECIFIED = 0
INSTANCE_TYPE_FILTER_ALL = 1
INSTANCE_TYPE_FILTER_CPU = 2
INSTANCE_TYPE_FILTER_GPU = 3
INSTANCE_TYPE_UNSPECIFIED = 0
INSTANCE_TYPE_CPU = 1
INSTANCE_TYPE_GPU = 2


_INSTANCETEMPLATESPEC = DESCRIPTOR.message_types_by_name['InstanceTemplateSpec']
_INSTANCEFILTER = DESCRIPTOR.message_types_by_name['InstanceFilter']
InstanceTemplateSpec = _reflection.GeneratedProtocolMessageType('InstanceTemplateSpec', (_message.Message,), {
  'DESCRIPTOR' : _INSTANCETEMPLATESPEC,
  '__module__' : 'qwak.instance_template.instance_template_pb2'
  # @@protoc_insertion_point(class_scope:qwak.management.instance_template.InstanceTemplateSpec)
  })
_sym_db.RegisterMessage(InstanceTemplateSpec)

InstanceFilter = _reflection.GeneratedProtocolMessageType('InstanceFilter', (_message.Message,), {
  'DESCRIPTOR' : _INSTANCEFILTER,
  '__module__' : 'qwak.instance_template.instance_template_pb2'
  # @@protoc_insertion_point(class_scope:qwak.management.instance_template.InstanceFilter)
  })
_sym_db.RegisterMessage(InstanceFilter)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n,com.qwak.ai.management.instance_template.apiP\001'
  _INSTANCETYPEFILTER._serialized_start=545
  _INSTANCETYPEFILTER._serialized_end=693
  _INSTANCETYPE._serialized_start=695
  _INSTANCETYPE._serialized_end=786
  _INSTANCETEMPLATESPEC._serialized_start=135
  _INSTANCETEMPLATESPEC._serialized_end=439
  _INSTANCEFILTER._serialized_start=441
  _INSTANCEFILTER._serialized_end=542
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _CommunicationManagerType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CommunicationManagerTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CommunicationManagerType.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    INVALID_MANAGER: _CommunicationManagerType.ValueType  # 0
    DIRECT_MANAGER: _CommunicationManagerType.ValueType  # 1
    ADMIRAL_MANAGER: _CommunicationManagerType.ValueType  # 2

class CommunicationManagerType(_CommunicationManagerType, metaclass=_CommunicationManagerTypeEnumTypeWrapper): ...

INVALID_MANAGER: CommunicationManagerType.ValueType  # 0
DIRECT_MANAGER: CommunicationManagerType.ValueType  # 1
ADMIRAL_MANAGER: CommunicationManagerType.ValueType  # 2
global___CommunicationManagerType = CommunicationManagerType

class QwakEnvironmentConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EDGE_SERVICES_URL_FIELD_NUMBER: builtins.int
    MANAGEMENT_ACCESS_URL_FIELD_NUMBER: builtins.int
    MODEL_API_URL_FIELD_NUMBER: builtins.int
    ANALYTICS_DATA_API_URL_FIELD_NUMBER: builtins.int
    OBJECT_STORAGE_BUCKET_FIELD_NUMBER: builtins.int
    BI_SYSTEM_API_KEY_FIELD_NUMBER: builtins.int
    CLOUD_CONFIGURATION_FIELD_NUMBER: builtins.int
    FEATURE_STORE_CONFIGURATION_FIELD_NUMBER: builtins.int
    CONTAINER_REGISTRY_PREFIX_FIELD_NUMBER: builtins.int
    COMMUNICATION_MANAGER_TYPE_FIELD_NUMBER: builtins.int
    edge_services_url: builtins.str
    """The Grpc service url on edge"""
    management_access_url: builtins.str
    """The management url of inter qwak services communication"""
    model_api_url: builtins.str
    """The endpoint of the models of actions like inference"""
    analytics_data_api_url: builtins.str
    """The endpoint which the analytics data is send to"""
    object_storage_bucket: builtins.str
    """The object storage bucket name of the environment"""
    bi_system_api_key: builtins.str
    """The BI system token of the environment"""
    @property
    def cloud_configuration(self) -> global___QwakCloudConfiguration:
        """Configuration specific for the environment hosting cloud"""
    @property
    def feature_store_configuration(self) -> global___FeatureStoreConfiguration:
        """Feature store configuration"""
    container_registry_prefix: builtins.str
    """The docker registry prefix, we got it with high number because we might want to remove it in the near future."""
    communication_manager_type: global___CommunicationManagerType.ValueType
    """Qwak communication type"""
    def __init__(
        self,
        *,
        edge_services_url: builtins.str = ...,
        management_access_url: builtins.str = ...,
        model_api_url: builtins.str = ...,
        analytics_data_api_url: builtins.str = ...,
        object_storage_bucket: builtins.str = ...,
        bi_system_api_key: builtins.str = ...,
        cloud_configuration: global___QwakCloudConfiguration | None = ...,
        feature_store_configuration: global___FeatureStoreConfiguration | None = ...,
        container_registry_prefix: builtins.str = ...,
        communication_manager_type: global___CommunicationManagerType.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cloud_configuration", b"cloud_configuration", "feature_store_configuration", b"feature_store_configuration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["analytics_data_api_url", b"analytics_data_api_url", "bi_system_api_key", b"bi_system_api_key", "cloud_configuration", b"cloud_configuration", "communication_manager_type", b"communication_manager_type", "container_registry_prefix", b"container_registry_prefix", "edge_services_url", b"edge_services_url", "feature_store_configuration", b"feature_store_configuration", "management_access_url", b"management_access_url", "model_api_url", b"model_api_url", "object_storage_bucket", b"object_storage_bucket"]) -> None: ...

global___QwakEnvironmentConfiguration = QwakEnvironmentConfiguration

class FeatureStoreConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AIRFLOW_CONNECTION_ID_FIELD_NUMBER: builtins.int
    REDIS_ADDRESS_FIELD_NUMBER: builtins.int
    USER_CODE_EXECUTION_ALLOWED_FIELD_NUMBER: builtins.int
    airflow_connection_id: builtins.str
    """The airflow connection id"""
    redis_address: builtins.str
    """The address to connect redis"""
    user_code_execution_allowed: builtins.bool
    """Boolean indicator is user code (deserializers, transformations) allowed to be executed
    in the feature store - this should be False for multi tenant SaaS envs (shared between
    multiple customers)
    """
    def __init__(
        self,
        *,
        airflow_connection_id: builtins.str = ...,
        redis_address: builtins.str = ...,
        user_code_execution_allowed: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["airflow_connection_id", b"airflow_connection_id", "redis_address", b"redis_address", "user_code_execution_allowed", b"user_code_execution_allowed"]) -> None: ...

global___FeatureStoreConfiguration = FeatureStoreConfiguration

class QwakCloudConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AWS_CLOUD_CONFIGURATION_FIELD_NUMBER: builtins.int
    GCP_CLOUD_CONFIGURATION_FIELD_NUMBER: builtins.int
    @property
    def aws_cloud_configuration(self) -> global___AwsCloudConfiguration:
        """Configuration specific to AWS cloud provider"""
    @property
    def gcp_cloud_configuration(self) -> global___GcpCloudConfiguration:
        """Configuration specific to GCP cloud provider"""
    def __init__(
        self,
        *,
        aws_cloud_configuration: global___AwsCloudConfiguration | None = ...,
        gcp_cloud_configuration: global___GcpCloudConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["aws_cloud_configuration", b"aws_cloud_configuration", "configuration", b"configuration", "gcp_cloud_configuration", b"gcp_cloud_configuration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aws_cloud_configuration", b"aws_cloud_configuration", "configuration", b"configuration", "gcp_cloud_configuration", b"gcp_cloud_configuration"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["configuration", b"configuration"]) -> typing_extensions.Literal["aws_cloud_configuration", "gcp_cloud_configuration"] | None: ...

global___QwakCloudConfiguration = QwakCloudConfiguration

class AwsCloudConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REGION_FIELD_NUMBER: builtins.int
    ROLE_ARN_FIELD_NUMBER: builtins.int
    FEATURE_STORE_EMR_FIELD_NUMBER: builtins.int
    EXTERNAL_ID_FIELD_NUMBER: builtins.int
    WORKGROUP_FIELD_NUMBER: builtins.int
    region: builtins.str
    """The region of the environment"""
    role_arn: builtins.str
    """The role ARN which qwak platform perform actions on behalf"""
    @property
    def feature_store_emr(self) -> global___FeatureStoreEmr:
        """Feature store emr configuration"""
    external_id: builtins.str
    workgroup: builtins.str
    def __init__(
        self,
        *,
        region: builtins.str = ...,
        role_arn: builtins.str = ...,
        feature_store_emr: global___FeatureStoreEmr | None = ...,
        external_id: builtins.str = ...,
        workgroup: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["feature_store_emr", b"feature_store_emr"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["external_id", b"external_id", "feature_store_emr", b"feature_store_emr", "region", b"region", "role_arn", b"role_arn", "workgroup", b"workgroup"]) -> None: ...

global___AwsCloudConfiguration = AwsCloudConfiguration

class GcpCloudConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_ID_FIELD_NUMBER: builtins.int
    REGION_FIELD_NUMBER: builtins.int
    BIG_LAKE_CONNECTION_NAME_FIELD_NUMBER: builtins.int
    BIG_LAKE_LOCATION_FIELD_NUMBER: builtins.int
    SERVICE_ACCOUNT_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    """The project id of the environment"""
    region: builtins.str
    """The region of the environment"""
    big_lake_connection_name: builtins.str
    big_lake_location: builtins.str
    """The location of the big lake a.k.a region"""
    service_account: builtins.str
    """The service account that qwak platform perform actions on behalf"""
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
        region: builtins.str = ...,
        big_lake_connection_name: builtins.str = ...,
        big_lake_location: builtins.str = ...,
        service_account: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["big_lake_connection_name", b"big_lake_connection_name", "big_lake_location", b"big_lake_location", "project_id", b"project_id", "region", b"region", "service_account", b"service_account"]) -> None: ...

global___GcpCloudConfiguration = GcpCloudConfiguration

class FeatureStoreEmr(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EC2_KEY_NAME_FIELD_NUMBER: builtins.int
    EC2_SUBNET_ID_FIELD_NUMBER: builtins.int
    EMR_JOB_FLOW_ROLE_FIELD_NUMBER: builtins.int
    EMR_SERVICE_ROLE_FIELD_NUMBER: builtins.int
    EC2_ADDITIONAL_SECURITY_GROUP_FIELD_NUMBER: builtins.int
    TARGET_ON_DEMAND_CAPACITY_WORKERS_FIELD_NUMBER: builtins.int
    ec2_key_name: builtins.str
    """EC2 key name for ssh into the EMR nodes"""
    ec2_subnet_id: builtins.str
    """The subnet id for ec2 running the EMR"""
    emr_job_flow_role: builtins.str
    """Aws role for EMR workers"""
    emr_service_role: builtins.str
    """Aws role for the emr service orchestrator"""
    ec2_additional_security_group: builtins.str
    """Additional security groups for EMR nodes (EC2)"""
    target_on_demand_capacity_workers: builtins.int
    """Target on demand capacity workers requested for EMR"""
    def __init__(
        self,
        *,
        ec2_key_name: builtins.str = ...,
        ec2_subnet_id: builtins.str = ...,
        emr_job_flow_role: builtins.str = ...,
        emr_service_role: builtins.str = ...,
        ec2_additional_security_group: builtins.str = ...,
        target_on_demand_capacity_workers: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ec2_additional_security_group", b"ec2_additional_security_group", "ec2_key_name", b"ec2_key_name", "ec2_subnet_id", b"ec2_subnet_id", "emr_job_flow_role", b"emr_job_flow_role", "emr_service_role", b"emr_service_role", "target_on_demand_capacity_workers", b"target_on_demand_capacity_workers"]) -> None: ...

global___FeatureStoreEmr = FeatureStoreEmr



import os

from qwak.inner.build_logic.interface.step_inteface import Step
from qwak.inner.build_logic.tools.files import cleaning_up_after_build


class CleanupStep(Step):
    STEP_DESCRIPTION = "Clean build artifacts"

    def description(self) -> str:
        return self.STEP_DESCRIPTION

    def execute(self) -> None:
        self.build_logger.debug(
            "Cleanup environment - Deleting file and intermediate images"
        )
        if os.getenv("POD_NAME"):
            self.build_logger.debug("Skipping cleanup - Remote executor is temporary")
        else:
            cleaning_up_after_build(self)



import os
from functools import partial
from math import ceil
from multiprocessing import Pool, set_start_method
from typing import Dict, Iterable, List, Optional, Tuple, TypeVar, Union

from _qwak_proto.qwak.vectors.v1.vector_pb2 import (
    DoubleVector,
    Property,
    StoredVector,
    VectorIdentifier,
)
from dependency_injector.wiring import Provide, inject
from qwak.clients.vector_store.serving_client import VectorServingClient
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer
from qwak.vector_store.inference_client import VectorStoreInferenceClient
from typeguard import typechecked

_NaturalInput = TypeVar("T")
_NaturalInputs = List[_NaturalInput]
_Vector = List[float]
_Properties = Dict[str, Union[str, int, bool, float]]

_type_to_proto_property_mapping: Dict = {
    str: "string_val",
    bool: "bool_val",
    int: "int_val",
    float: "double_val",
}


def _build_property(key: str, value: Union[str, int, bool, float]):
    type_val = _type_to_proto_property_mapping.get(type(value), None)
    if not type_val:
        raise QwakException(
            f"Cannot upsert vector with property value type {type(value)}. "
            f"Supported types are: {list(_type_to_proto_property_mapping.keys())}"
        )

    property_args = {"name": key, type_val: value}
    return Property(**property_args)


def _rewire_qwak_container(config):
    # re-creating the container using the config from the original container.
    # note that this runs in a fresh interpreter - at that point there's a running
    # container because of the imports, but it won't necessarily have the same config
    # as the container in the parent process that spawned this one.
    # rewiring only the vector store + ecosystem and authentication - if using stuff from
    # other modules pls feel free to add it here.
    new_container = QwakContainer(config=config)
    from qwak.clients import vector_store
    from qwak.clients.administration import authentication, eco_system

    new_container.wire(
        packages=[
            authentication,
            eco_system,
            vector_store,
        ]
    )


@typechecked
def _divide_chunks(lst: List, chunk_size: int):
    if chunk_size <= 0:
        raise QwakException("Chunk size must be a positive integer")

    num_items: int = len(lst)
    num_chunks: int = ceil(num_items / chunk_size)
    for i in range(num_chunks):
        yield lst[i * chunk_size : (i + 1) * chunk_size]


def _get_vector_identifier(t: Union[str, Tuple[str, str]]):
    if type(t) is str:
        return VectorIdentifier(vector_id=t)
    return VectorIdentifier(vector_id=t[0], tenant_id=t[1])


def _upsert_vector_block(
    vector_tuples: List[Tuple[Union[str, Tuple[str, str]], _Vector, _Properties]],
    chunk_size: int,
    collection_name: str,
    edge_services_url: str,
) -> None:
    vector_serving_client: VectorServingClient = VectorServingClient(
        edge_services_url=edge_services_url
    )
    for chunk in _divide_chunks(vector_tuples, chunk_size):
        # chunk is a list of (id, vector, properties) tuples
        vector_serving_client.upsert_vectors(
            collection_name=collection_name,
            vectors=[
                StoredVector(
                    vector_identifier=_get_vector_identifier(tpl[0]),
                    vector=DoubleVector(element=tpl[1]),
                    properties=[
                        _build_property(key, value) for (key, value) in tpl[2].items()
                    ],
                )
                for tpl in chunk
            ],
        )


def _upsert_natural_input_block(
    vector_tuples: List[Tuple[VectorIdentifier, _NaturalInput, _Properties]],
    chunk_size: int,
    vectorizer_name: str,
    collection_name: str,
    edge_services_url: str,
) -> None:
    vector_serving_client: VectorServingClient = VectorServingClient(
        edge_services_url=edge_services_url
    )
    inference_client: VectorStoreInferenceClient = VectorStoreInferenceClient(
        model_id=vectorizer_name
    )
    for chunk in _divide_chunks(vector_tuples, chunk_size):
        # chunk is a list of (id, _NaturalInput, properties) tuples
        vector_serving_client.upsert_vectors(
            collection_name=collection_name,
            vectors=[
                StoredVector(
                    vector_identifier=_get_vector_identifier(tpl[0]),
                    vector=DoubleVector(
                        element=inference_client.get_embedding(natural_input=tpl[1])
                    ),
                    properties=[
                        _build_property(key, value) for (key, value) in tpl[2].items()
                    ],
                )
                for tpl in chunk
            ],
        )


@inject
def _upsert_natural_input(
    vector_tuples: List[Tuple[Union[str, Tuple[str, str]], _NaturalInput, _Properties]],
    chunk_size: int,
    vectorizer_name: str,
    collection_name: str,
    edge_services_url: str,
    multiproc: bool = False,
    max_processes: Optional[int] = None,
    config=Provide[QwakContainer.config],
):
    if not multiproc:
        _upsert_natural_input_block(
            vector_tuples=vector_tuples,
            chunk_size=chunk_size,
            vectorizer_name=vectorizer_name,
            collection_name=collection_name,
            edge_services_url=edge_services_url,
        )
    else:
        if max_processes is None:
            max_processes = os.cpu_count()
        effective_block_size: int = ceil(len(vector_tuples) / (max_processes * 4))

        f = partial(
            _upsert_natural_input_block,
            chunk_size=chunk_size,
            vectorizer_name=vectorizer_name,
            collection_name=collection_name,
            edge_services_url=edge_services_url,
        )

        blocks: Iterable[List[Tuple[str, _NaturalInput, _Properties]]] = _divide_chunks(
            vector_tuples, effective_block_size
        )
        initializer = partial(_rewire_qwak_container, config=config)
        set_start_method("spawn", force=True)

        with Pool(processes=max_processes, initializer=initializer) as p:
            p.map(f, blocks)


@inject
def _upsert_vectors(
    vector_tuples: List[Tuple[Union[str, Tuple[str, str]], _Vector, _Properties]],
    chunk_size: int,
    collection_name: str,
    edge_services_url: str,
    multiproc: bool = False,
    max_processes: Optional[int] = None,
    config=Provide[QwakContainer.config],
):
    if not multiproc:
        _upsert_vector_block(
            vector_tuples=vector_tuples,
            chunk_size=chunk_size,
            collection_name=collection_name,
            edge_services_url=edge_services_url,
        )
    else:
        if max_processes is None:
            max_processes = os.cpu_count()
        effective_block_size: int = ceil(len(vector_tuples) / (max_processes * 4))

        f = partial(
            _upsert_vector_block,
            chunk_size=chunk_size,
            collection_name=collection_name,
            edge_services_url=edge_services_url,
        )
        blocks: Iterable[List[Tuple[str, _Vector, _Properties]]] = list(
            _divide_chunks(vector_tuples, effective_block_size)
        )

        set_start_method("spawn", force=True)
        initializer = partial(_rewire_qwak_container, config=config)
        with Pool(processes=max_processes, initializer=initializer) as p:
            p.map(f, blocks)



from dataclasses import dataclass, field

from _qwak_proto.qwak.file_versioning.file_versioning_pb2 import (
    FileTagSpec as FileTagSpecProto,
)


@dataclass
class FileTag:
    model_id: str = field(default=None)
    build_id: str = field(default=None)
    tag: str = field(default=None)
    extension_type: str = field(default=None)
    environment_id: str = field(default=None)

    @staticmethod
    def from_proto(file_versioning_spec_proto: FileTagSpecProto):
        return FileTag(
            model_id=file_versioning_spec_proto.model_id,
            build_id=file_versioning_spec_proto.build_id,
            tag=file_versioning_spec_proto.tag,
            extension_type=file_versioning_spec_proto.extension_type,
            environment_id=file_versioning_spec_proto.environment_id,
        )



import json

from qwak.clients.administration.eco_system.client import EcosystemClient
from qwak.exceptions import QwakException, QwakHTTPException
from qwak.vector_store.rest_helpers import RestSession


class VectorStoreInferenceClient:
    def __init__(
        self,
        model_id: str,
    ):
        """
        :param model_id: The model id to invoke against.
        """

        self.r_session = RestSession()
        self.ecosystem_client = EcosystemClient()

        authenticated_user_context = (
            self.ecosystem_client.get_authenticated_user_context()
        )
        account_details = authenticated_user_context.user.account_details
        default_environment_id = account_details.default_environment_id
        model_url_prefix = account_details.environment_by_id[
            default_environment_id
        ].configuration.model_api_url

        self.model_id = model_id
        self.content_type = "application/json; format=pandas-split"
        self.model_api = _get_model_url(
            model_id=model_id, model_url_prefix=model_url_prefix
        )

    def predict(
        self,
        feature_vectors,
    ):
        """
        Perform a prediction request against a Qwak based model

        :param feature_vectors: A list of feature vectors to predict against. Each feature vector is modeled as a python
         dictionary
        :return: Prediction response from the model
        """

        if feature_vectors.__class__.__name__ == "DataFrame":
            feature_vectors = feature_vectors.to_json(orient="split")

        if isinstance(feature_vectors, dict) or isinstance(feature_vectors, list):
            feature_vectors = json.dumps(feature_vectors)

        try:
            response = self.r_session.post(
                self.model_api, data=feature_vectors, headers={}
            )

            if response.status_code >= 400:
                exception_class_name = response.headers.get("X-Exception-Class")
                msg = f"{response.status_code}: {response.text}"
                raise QwakHTTPException(response.status_code, msg, exception_class_name)

            elif response.status_code != 200:
                raise QwakHTTPException(response.status_code, response.content)

            dict_response = json.loads(response.content)
            return dict_response
        except QwakHTTPException as e:
            raise e
        except Exception as e:
            raise RuntimeError(f"Failed to make a prediction request. Error is: {e}")

    def get_embedding(self, natural_input: str):
        feature_vector = [
            {
                "input": natural_input,
            }
        ]
        try:
            result_list = self.predict(feature_vector)
        except Exception as e:
            raise QwakException(
                f"Vectorizer {self.model_id} failed to transform input {feature_vector} to vectors. Error is: {str(e)}"
            )
        try:
            vector = result_list[0]["embeddings"]
        except Exception:
            raise QwakException(
                f"Vectorizer {self.model_id} must return a dataframe containing an 'embeddings' column"
            )
        if not vector:
            raise QwakException(
                f"Vectorizer {self.model_id} did not return embeddings for the given natural input. Unable to continue with the query."
            )
        return vector


def _get_model_url(model_id: str, model_url_prefix: str) -> str:
    scheme = "http" if model_url_prefix.startswith("localhost") else "https"
    effective_model_id = model_id.replace("-", "_")
    return (
        f"{scheme}://{model_url_prefix}/v1/{effective_model_id}/predict"  # noqa: E231
    )



from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
    SnowflakeSource as ProtoSnowflakeSource,
)
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from qwak.exceptions import QwakException
from qwak.feature_store.data_sources.batch._batch import BaseBatchSource


@dataclass
class SnowflakeSource(BaseBatchSource):
    host: str
    username_secret_name: str
    password_secret_name: str
    database: str
    schema: str
    warehouse: str
    table: Optional[str] = None
    query: Optional[str] = None
    repository: Optional[str] = None

    def __post_init__(self):
        self._validate()

    def _validate(self):
        if self.table and self.query:
            raise QwakException("Only one of query and table may be set")
        if not self.table and not self.query:
            raise QwakException("One of table or query must be set")

        if not self.username_secret_name:
            raise QwakException("username_secret_name must be set!")

        if not self.password_secret_name:
            raise QwakException("password_secret_name must be set!")

    def _to_proto(self, artifact_url: Optional[str] = None):
        return ProtoDataSourceSpec(
            featureset_repository_name=self.repository,
            batch_source=ProtoBatchSource(
                name=self.name,
                description=self.description,
                date_created_column=self.date_created_column,
                snowflakeSource=ProtoSnowflakeSource(
                    host=self.host,
                    username_secret_name=self.username_secret_name,
                    password_secret_name=self.password_secret_name,
                    database=self.database,
                    schema=self.schema,
                    warehouse=self.warehouse,
                    table=self.table,
                    query=self.query,
                ),
            ),
        )

    @classmethod
    def _from_proto(cls, proto):
        snowflake = proto.snowflakeSource
        return cls(
            name=proto.name,
            date_created_column=proto.date_created_column,
            description=proto.description,
            host=snowflake.host,
            username_secret_name=snowflake.username_secret_name,
            password_secret_name=snowflake.password_secret_name,
            database=snowflake.database,
            schema=snowflake.schema,
            warehouse=snowflake.warehouse,
            table=snowflake.table,
            query=snowflake.query,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/analytics/analytics.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1eqwak/analytics/analytics.proto\x12\x0eqwak.analytics\"w\n\nResultData\x12\x0f\n\x07headers\x18\x01 \x03(\t\x12/\n\x0bheadersDesc\x18\x03 \x03(\x0b\x32\x1a.qwak.analytics.HeaderDesc\x12\'\n\x04rows\x18\x02 \x03(\x0b\x32\x19.qwak.analytics.ResultRow\"(\n\nHeaderDesc\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\"\x1b\n\tResultRow\x12\x0e\n\x06values\x18\x01 \x03(\t\":\n\x0bTableColumn\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0f\n\x07\x63omment\x18\x03 \x01(\t\"O\n\x15TableColumnDefinition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12(\n\x04type\x18\x02 \x01(\x0e\x32\x1a.qwak.analytics.ColumnType\"\xe8\x01\n\x0fTableDefinition\x12\x0f\n\x07\x64\x62_name\x18\x01 \x01(\t\x12\x12\n\ntable_name\x18\x02 \x01(\t\x12\x36\n\x07\x63olumns\x18\x03 \x03(\x0b\x32%.qwak.analytics.TableColumnDefinition\x12\x11\n\tdata_path\x18\x04 \x01(\t\x12*\n\x06\x66ormat\x18\x05 \x01(\x0b\x32\x1a.qwak.analytics.DataFormat\x12\x39\n\npartitions\x18\x06 \x03(\x0b\x32%.qwak.analytics.TableColumnDefinition\"?\n\nDataFormat\x12\'\n\x03\x63sv\x18\x01 \x01(\x0b\x32\x18.qwak.analytics.CsvInputH\x00\x42\x08\n\x06\x66ormat\"F\n\x08\x43svInput\x12\x11\n\tdelimiter\x18\x01 \x01(\t\x12\x12\n\nquote_char\x18\x02 \x01(\t\x12\x13\n\x0b\x65scape_char\x18\x03 \x01(\t\"0\n\x1cQueryResultDownloadURLParams\x12\x10\n\x08query_id\x18\x01 \x01(\t*N\n\x0bQueryStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\x0b\n\x07PENDING\x10\x02\x12\x0c\n\x08\x43\x41NCELED\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04*\xad\x01\n\nColumnType\x12\x10\n\x0cINVALID_TYPE\x10\x00\x12\x0b\n\x07\x42OOLEAN\x10\x01\x12\x0b\n\x07TINYINT\x10\x02\x12\x0c\n\x08SMALLINT\x10\x03\x12\x07\n\x03INT\x10\x04\x12\n\n\x06\x42IGINT\x10\x05\x12\n\n\x06\x44OUBLE\x10\x06\x12\t\n\x05\x46LOAT\x10\x07\x12\x08\n\x04\x43HAR\x10\x08\x12\n\n\x06STRING\x10\t\x12\n\n\x06\x42INARY\x10\n\x12\x08\n\x04\x44\x41TE\x10\x0b\x12\r\n\tTIMESTAMP\x10\x0c\x42\x86\x01\n\x19\x63om.qwak.ai.analytics.apiP\x01Zggithub.com/qwak-ai/qwak-platform/services/core/java/analytics/analytics-api/pb/qwak/analytics;analyticsb\x06proto3')

_QUERYSTATUS = DESCRIPTOR.enum_types_by_name['QueryStatus']
QueryStatus = enum_type_wrapper.EnumTypeWrapper(_QUERYSTATUS)
_COLUMNTYPE = DESCRIPTOR.enum_types_by_name['ColumnType']
ColumnType = enum_type_wrapper.EnumTypeWrapper(_COLUMNTYPE)
INVALID = 0
SUCCESS = 1
PENDING = 2
CANCELED = 3
FAILED = 4
INVALID_TYPE = 0
BOOLEAN = 1
TINYINT = 2
SMALLINT = 3
INT = 4
BIGINT = 5
DOUBLE = 6
FLOAT = 7
CHAR = 8
STRING = 9
BINARY = 10
DATE = 11
TIMESTAMP = 12


_RESULTDATA = DESCRIPTOR.message_types_by_name['ResultData']
_HEADERDESC = DESCRIPTOR.message_types_by_name['HeaderDesc']
_RESULTROW = DESCRIPTOR.message_types_by_name['ResultRow']
_TABLECOLUMN = DESCRIPTOR.message_types_by_name['TableColumn']
_TABLECOLUMNDEFINITION = DESCRIPTOR.message_types_by_name['TableColumnDefinition']
_TABLEDEFINITION = DESCRIPTOR.message_types_by_name['TableDefinition']
_DATAFORMAT = DESCRIPTOR.message_types_by_name['DataFormat']
_CSVINPUT = DESCRIPTOR.message_types_by_name['CsvInput']
_QUERYRESULTDOWNLOADURLPARAMS = DESCRIPTOR.message_types_by_name['QueryResultDownloadURLParams']
ResultData = _reflection.GeneratedProtocolMessageType('ResultData', (_message.Message,), {
  'DESCRIPTOR' : _RESULTDATA,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.ResultData)
  })
_sym_db.RegisterMessage(ResultData)

HeaderDesc = _reflection.GeneratedProtocolMessageType('HeaderDesc', (_message.Message,), {
  'DESCRIPTOR' : _HEADERDESC,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.HeaderDesc)
  })
_sym_db.RegisterMessage(HeaderDesc)

ResultRow = _reflection.GeneratedProtocolMessageType('ResultRow', (_message.Message,), {
  'DESCRIPTOR' : _RESULTROW,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.ResultRow)
  })
_sym_db.RegisterMessage(ResultRow)

TableColumn = _reflection.GeneratedProtocolMessageType('TableColumn', (_message.Message,), {
  'DESCRIPTOR' : _TABLECOLUMN,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.TableColumn)
  })
_sym_db.RegisterMessage(TableColumn)

TableColumnDefinition = _reflection.GeneratedProtocolMessageType('TableColumnDefinition', (_message.Message,), {
  'DESCRIPTOR' : _TABLECOLUMNDEFINITION,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.TableColumnDefinition)
  })
_sym_db.RegisterMessage(TableColumnDefinition)

TableDefinition = _reflection.GeneratedProtocolMessageType('TableDefinition', (_message.Message,), {
  'DESCRIPTOR' : _TABLEDEFINITION,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.TableDefinition)
  })
_sym_db.RegisterMessage(TableDefinition)

DataFormat = _reflection.GeneratedProtocolMessageType('DataFormat', (_message.Message,), {
  'DESCRIPTOR' : _DATAFORMAT,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.DataFormat)
  })
_sym_db.RegisterMessage(DataFormat)

CsvInput = _reflection.GeneratedProtocolMessageType('CsvInput', (_message.Message,), {
  'DESCRIPTOR' : _CSVINPUT,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.CsvInput)
  })
_sym_db.RegisterMessage(CsvInput)

QueryResultDownloadURLParams = _reflection.GeneratedProtocolMessageType('QueryResultDownloadURLParams', (_message.Message,), {
  'DESCRIPTOR' : _QUERYRESULTDOWNLOADURLPARAMS,
  '__module__' : 'qwak.analytics.analytics_pb2'
  # @@protoc_insertion_point(class_scope:qwak.analytics.QueryResultDownloadURLParams)
  })
_sym_db.RegisterMessage(QueryResultDownloadURLParams)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.qwak.ai.analytics.apiP\001Zggithub.com/qwak-ai/qwak-platform/services/core/java/analytics/analytics-api/pb/qwak/analytics;analytics'
  _QUERYSTATUS._serialized_start=805
  _QUERYSTATUS._serialized_end=883
  _COLUMNTYPE._serialized_start=886
  _COLUMNTYPE._serialized_end=1059
  _RESULTDATA._serialized_start=50
  _RESULTDATA._serialized_end=169
  _HEADERDESC._serialized_start=171
  _HEADERDESC._serialized_end=211
  _RESULTROW._serialized_start=213
  _RESULTROW._serialized_end=240
  _TABLECOLUMN._serialized_start=242
  _TABLECOLUMN._serialized_end=300
  _TABLECOLUMNDEFINITION._serialized_start=302
  _TABLECOLUMNDEFINITION._serialized_end=381
  _TABLEDEFINITION._serialized_start=384
  _TABLEDEFINITION._serialized_end=616
  _DATAFORMAT._serialized_start=618
  _DATAFORMAT._serialized_end=681
  _CSVINPUT._serialized_start=683
  _CSVINPUT._serialized_end=753
  _QUERYRESULTDOWNLOADURLPARAMS._serialized_start=755
  _QUERYRESULTDOWNLOADURLPARAMS._serialized_end=803
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
    CsvSource as ProtoCsvSource,
)
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from qwak.feature_store.data_sources.batch._batch import BaseBatchSource
from qwak.feature_store.data_sources.batch.filesystem.base_config import (
    FileSystemConfiguration,
)
from qwak.feature_store.data_sources.batch.filesystem.utils import (
    get_fs_config_from_proto,
)


@dataclass
class CsvSource(BaseBatchSource):
    path: str
    quote_character: str = '"'
    escape_character: str = '"'
    filesystem_configuration: Optional[FileSystemConfiguration] = None
    repository: Optional[str] = None

    @classmethod
    def _from_proto(cls, proto):
        csv = proto.csvSource

        fs_conf = get_fs_config_from_proto(csv.filesystem_configuration)

        return cls(
            name=proto.name,
            date_created_column=proto.date_created_column,
            description=proto.description,
            path=csv.path,
            quote_character=csv.quote_character,
            escape_character=csv.escape_character,
            filesystem_configuration=fs_conf,
        )

    def _to_proto(self, artifact_url: Optional[str] = None):
        fs_conf = None
        if self.filesystem_configuration:
            fs_conf = self.filesystem_configuration._to_proto()

        return ProtoDataSourceSpec(
            featureset_repository_name=self.repository,
            batch_source=ProtoBatchSource(
                name=self.name,
                description=self.description,
                date_created_column=self.date_created_column,
                csvSource=ProtoCsvSource(
                    path=self.path,
                    quote_character=self.quote_character,
                    escape_character=self.escape_character,
                    filesystem_configuration=fs_conf,
                ),
            ),
        )



import sys


class QuietError(Exception):
    # All who inherit me shall not traceback, but be spoken of cleanly
    def __init__(self, message):
        self.message = None

    def __str__(self):
        return f"\033[91m{self.message}\033[0m"


def quiet_hook(kind, message, traceback):
    if issubclass(kind, QuietError):
        print("{0}".format(message))  # Only print Error Type and Message
    else:
        sys.__excepthook__(
            kind, message, traceback
        )  # Print Error Type, Message and Traceback


sys.excepthook = quiet_hook



from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
    JdbcSource as ProtoJdbcSource,
    RedshiftSource as ProtoRedshiftSource,
)
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from qwak.exceptions import QwakException
from qwak.feature_store.data_sources.batch._jdbc import JdbcSource


@dataclass
class RedshiftSource(JdbcSource):
    iam_role_arn: Optional[str] = None
    db_user: Optional[str] = None

    # these options are not yet supported
    access_key: Optional[str] = None
    secret_access_key: Optional[str] = None
    query_group: Optional[str] = "_qwak_featurestore"

    repository: Optional[str] = None

    def _validate(self):
        authentication_methods = sum(
            [
                bool(self.access_key and self.secret_access_key),
                bool(self.iam_role_arn),
                bool(self.username_secret_name and self.password_secret_name),
            ]
        )
        if authentication_methods > 1:
            raise QwakException(
                "Only one connection method must be set, access key id and secret access key id, IAM role arn"
                "or user id secret name and password secret name"
            )
        if authentication_methods < 1:
            raise QwakException(
                "A connection method must be set, either access key id and secret access key id, IAM role arn"
                "or user id secret name and password secret name"
            )

    def _to_proto(self, artifact_url: Optional[str] = None):
        return ProtoDataSourceSpec(
            featureset_repository_name=self.repository,
            batch_source=ProtoBatchSource(
                name=self.name,
                description=self.description,
                date_created_column=self.date_created_column,
                jdbcSource=ProtoJdbcSource(
                    url=self.url,
                    username_secret_name=self.username_secret_name,
                    password_secret_name=self.password_secret_name,
                    db_table=self.db_table,
                    query=self.query,
                    redshiftSource=ProtoRedshiftSource(
                        db_user=self.db_user,
                        iam_role_arn=self.iam_role_arn,
                        access_key_id=self.access_key,
                        secret_access_key=self.secret_access_key,
                    ),
                ),
            ),
        )

    @classmethod
    def _from_proto(cls, proto: ProtoBatchSource):
        redshift = proto.jdbcSource
        return cls(
            name=proto.name,
            date_created_column=proto.date_created_column,
            description=proto.description,
            url=redshift.url,
            username_secret_name=redshift.username_secret_name,
            password_secret_name=redshift.password_secret_name,
            db_table=redshift.db_table,
            query=redshift.query,
            iam_role_arn=redshift.redshiftSource.iam_role_arn,
            db_user=redshift.redshiftSource.db_user,
        )



from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
    BigquerySource as ProtoBigquerySource,
)
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from qwak.exceptions import QwakException
from qwak.feature_store.data_sources.batch._batch import BaseBatchSource


@dataclass
class BigQuerySource(BaseBatchSource):
    credentials_secret_name: str
    project: Optional[str] = None
    dataset: Optional[str] = None
    table: Optional[str] = None
    parent_project: Optional[str] = None
    sql: Optional[str] = None
    views_enabled: Optional[bool] = False
    materialization_dataset: Optional[str] = None
    materialization_project: Optional[str] = None
    repository: Optional[str] = None

    def __post_init__(self):
        if not (self.sql or (self.table and self.dataset)):
            raise QwakException("Either SQL or (table and dataset) must be provided.")

        if self.sql:
            if not self.materialization_project:
                raise QwakException(
                    "For SQL query mode, materialization project setting must be set."
                )
            if not self.materialization_dataset:
                raise QwakException(
                    "For SQL query mode, materialization dataset setting must be set."
                )
            self.views_enabled = True

    @classmethod
    def _from_proto(cls, proto):
        bigquery = proto.bigquerySource
        return cls(
            name=proto.name,
            date_created_column=proto.date_created_column,
            description=proto.description,
            credentials_secret_name=bigquery.credentials_secret_name,
            dataset=bigquery.dataset,
            table=bigquery.table,
            project=bigquery.project,
            parent_project=bigquery.parent_project,
            sql=bigquery.sql,
            views_enabled=bigquery.views_enabled,
            materialization_project=bigquery.materialization_project,
            materialization_dataset=bigquery.materialization_dataset,
        )

    def _to_proto(self, artifact_url: Optional[str] = None):
        return ProtoDataSourceSpec(
            featureset_repository_name=self.repository,
            batch_source=ProtoBatchSource(
                name=self.name,
                description=self.description,
                date_created_column=self.date_created_column,
                bigquerySource=ProtoBigquerySource(
                    credentials_secret_name=self.credentials_secret_name,
                    dataset=self.dataset,
                    table=self.table,
                    project=self.project,
                    parent_project=self.parent_project,
                    sql=self.sql,
                    views_enabled=self.views_enabled,
                    materialization_dataset=self.materialization_dataset,
                    materialization_project=self.materialization_project,
                ),
            ),
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.ecosystem.v0 import ecosystem_runtime_service_pb2 as qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2


class QwakEcosystemRuntimeStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Authenticate = channel.unary_unary(
                '/qwak.ecosystem.v0.QwakEcosystemRuntime/Authenticate',
                request_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.AuthenticateRequest.SerializeToString,
                response_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.AuthenticateResponse.FromString,
                )
        self.GetAuthenticatedUserContext = channel.unary_unary(
                '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetAuthenticatedUserContext',
                request_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAuthenticatedUserContextRequest.SerializeToString,
                response_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAuthenticatedUserContextResponse.FromString,
                )
        self.ListUserAccounts = channel.unary_unary(
                '/qwak.ecosystem.v0.QwakEcosystemRuntime/ListUserAccounts',
                request_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.ListUserAccountsRequest.SerializeToString,
                response_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.ListUserAccountsResponse.FromString,
                )
        self.GetAccount = channel.unary_unary(
                '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetAccount',
                request_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAccountRequest.SerializeToString,
                response_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAccountResponse.FromString,
                )
        self.GetCloudCredentials = channel.unary_unary(
                '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetCloudCredentials',
                request_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetCloudCredentialsRequest.SerializeToString,
                response_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetCloudCredentialsResponse.FromString,
                )
        self.GetUnifiedUserContext = channel.unary_unary(
                '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetUnifiedUserContext',
                request_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetUnifiedUserContextRequest.SerializeToString,
                response_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetUnifiedUserContextResponse.FromString,
                )


class QwakEcosystemRuntimeServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Authenticate(self, request, context):
        """Authenticate user and get token
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAuthenticatedUserContext(self, request, context):
        """List accounts of the user
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListUserAccounts(self, request, context):
        """List the accounts of the login user
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAccount(self, request, context):
        """Get account by id
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCloudCredentials(self, request, context):
        """Get cloud credentials
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetUnifiedUserContext(self, request, context):
        """Get a user context for a given user (from all kinds) and a given account
        That method is for internal use to get information by the services, and not for the end user.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QwakEcosystemRuntimeServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Authenticate': grpc.unary_unary_rpc_method_handler(
                    servicer.Authenticate,
                    request_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.AuthenticateRequest.FromString,
                    response_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.AuthenticateResponse.SerializeToString,
            ),
            'GetAuthenticatedUserContext': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAuthenticatedUserContext,
                    request_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAuthenticatedUserContextRequest.FromString,
                    response_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAuthenticatedUserContextResponse.SerializeToString,
            ),
            'ListUserAccounts': grpc.unary_unary_rpc_method_handler(
                    servicer.ListUserAccounts,
                    request_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.ListUserAccountsRequest.FromString,
                    response_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.ListUserAccountsResponse.SerializeToString,
            ),
            'GetAccount': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAccount,
                    request_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAccountRequest.FromString,
                    response_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAccountResponse.SerializeToString,
            ),
            'GetCloudCredentials': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCloudCredentials,
                    request_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetCloudCredentialsRequest.FromString,
                    response_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetCloudCredentialsResponse.SerializeToString,
            ),
            'GetUnifiedUserContext': grpc.unary_unary_rpc_method_handler(
                    servicer.GetUnifiedUserContext,
                    request_deserializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetUnifiedUserContextRequest.FromString,
                    response_serializer=qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetUnifiedUserContextResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.ecosystem.v0.QwakEcosystemRuntime', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class QwakEcosystemRuntime(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Authenticate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.ecosystem.v0.QwakEcosystemRuntime/Authenticate',
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.AuthenticateRequest.SerializeToString,
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.AuthenticateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAuthenticatedUserContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetAuthenticatedUserContext',
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAuthenticatedUserContextRequest.SerializeToString,
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAuthenticatedUserContextResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListUserAccounts(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.ecosystem.v0.QwakEcosystemRuntime/ListUserAccounts',
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.ListUserAccountsRequest.SerializeToString,
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.ListUserAccountsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAccount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetAccount',
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAccountRequest.SerializeToString,
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetAccountResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCloudCredentials(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetCloudCredentials',
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetCloudCredentialsRequest.SerializeToString,
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetCloudCredentialsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetUnifiedUserContext(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.ecosystem.v0.QwakEcosystemRuntime/GetUnifiedUserContext',
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetUnifiedUserContextRequest.SerializeToString,
            qwak_dot_ecosystem_dot_v0_dot_ecosystem__runtime__service__pb2.GetUnifiedUserContextResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _FeatureSetLastExecutionState:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FeatureSetLastExecutionStateEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FeatureSetLastExecutionState.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _FeatureSetLastExecutionState.ValueType  # 0
    FAILED: _FeatureSetLastExecutionState.ValueType  # 1
    SUCCESS: _FeatureSetLastExecutionState.ValueType  # 2
    INVALIDATING: _FeatureSetLastExecutionState.ValueType  # 3
    SUBMITTED: _FeatureSetLastExecutionState.ValueType  # 4
    RUNNING: _FeatureSetLastExecutionState.ValueType  # 5
    COMPLETED: _FeatureSetLastExecutionState.ValueType  # 6
    SUBMISSION_FAILED: _FeatureSetLastExecutionState.ValueType  # 7
    PENDING_RERUN: _FeatureSetLastExecutionState.ValueType  # 8
    SUCCEEDING: _FeatureSetLastExecutionState.ValueType  # 9
    FAILING: _FeatureSetLastExecutionState.ValueType  # 10
    PENDING: _FeatureSetLastExecutionState.ValueType  # 11

class FeatureSetLastExecutionState(_FeatureSetLastExecutionState, metaclass=_FeatureSetLastExecutionStateEnumTypeWrapper):
    """todo rename to FeatureSetExecutionState after the old state reporter deprication"""

UNKNOWN: FeatureSetLastExecutionState.ValueType  # 0
FAILED: FeatureSetLastExecutionState.ValueType  # 1
SUCCESS: FeatureSetLastExecutionState.ValueType  # 2
INVALIDATING: FeatureSetLastExecutionState.ValueType  # 3
SUBMITTED: FeatureSetLastExecutionState.ValueType  # 4
RUNNING: FeatureSetLastExecutionState.ValueType  # 5
COMPLETED: FeatureSetLastExecutionState.ValueType  # 6
SUBMISSION_FAILED: FeatureSetLastExecutionState.ValueType  # 7
PENDING_RERUN: FeatureSetLastExecutionState.ValueType  # 8
SUCCEEDING: FeatureSetLastExecutionState.ValueType  # 9
FAILING: FeatureSetLastExecutionState.ValueType  # 10
PENDING: FeatureSetLastExecutionState.ValueType  # 11
global___FeatureSetLastExecutionState = FeatureSetLastExecutionState

class FeatureSetState(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_SET_NAME_FIELD_NUMBER: builtins.int
    FEATURE_SET_ID_FIELD_NUMBER: builtins.int
    SPARK_APP_STATE_FIELD_NUMBER: builtins.int
    EXECUTOR_STATES_FIELD_NUMBER: builtins.int
    FEATURE_SET_JOB_TYPE_FIELD_NUMBER: builtins.int
    ERROR_MESSAGE_FIELD_NUMBER: builtins.int
    SPARK_APP_ID_FIELD_NUMBER: builtins.int
    LAST_MODIFIED_AT_FIELD_NUMBER: builtins.int
    RUN_ID_FIELD_NUMBER: builtins.int
    feature_set_name: builtins.str
    """Feature set name"""
    feature_set_id: builtins.str
    """Feature set id"""
    spark_app_state: global___FeatureSetLastExecutionState.ValueType
    """Spark application state for the feature set"""
    @property
    def executor_states(self) -> global___ExecutorStates:
        """List of executors state"""
    @property
    def feature_set_job_type(self) -> global___FeatureSetJobType:
        """The type of the feature set job"""
    error_message: builtins.str
    """Error message if exists"""
    spark_app_id: builtins.str
    """Spark application id"""
    @property
    def last_modified_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Last modification time of alerting channel"""
    run_id: builtins.int
    """Running id which represent the schedule job id"""
    def __init__(
        self,
        *,
        feature_set_name: builtins.str = ...,
        feature_set_id: builtins.str = ...,
        spark_app_state: global___FeatureSetLastExecutionState.ValueType = ...,
        executor_states: global___ExecutorStates | None = ...,
        feature_set_job_type: global___FeatureSetJobType | None = ...,
        error_message: builtins.str = ...,
        spark_app_id: builtins.str = ...,
        last_modified_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        run_id: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["executor_states", b"executor_states", "feature_set_job_type", b"feature_set_job_type", "last_modified_at", b"last_modified_at"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["error_message", b"error_message", "executor_states", b"executor_states", "feature_set_id", b"feature_set_id", "feature_set_job_type", b"feature_set_job_type", "feature_set_name", b"feature_set_name", "last_modified_at", b"last_modified_at", "run_id", b"run_id", "spark_app_id", b"spark_app_id", "spark_app_state", b"spark_app_state"]) -> None: ...

global___FeatureSetState = FeatureSetState

class FeatureSetJobType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAMING_FIELD_NUMBER: builtins.int
    BATCH_V1_FIELD_NUMBER: builtins.int
    STREAMING_AGGREGATION_FIELD_NUMBER: builtins.int
    DELETION_JOB_FIELD_NUMBER: builtins.int
    @property
    def streaming(self) -> global___Streaming: ...
    @property
    def batch_v1(self) -> global___BatchV1: ...
    @property
    def streaming_aggregation(self) -> global___StreamingAggregation: ...
    @property
    def deletion_job(self) -> global___DeletionJob: ...
    def __init__(
        self,
        *,
        streaming: global___Streaming | None = ...,
        batch_v1: global___BatchV1 | None = ...,
        streaming_aggregation: global___StreamingAggregation | None = ...,
        deletion_job: global___DeletionJob | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_v1", b"batch_v1", "deletion_job", b"deletion_job", "job_type", b"job_type", "streaming", b"streaming", "streaming_aggregation", b"streaming_aggregation"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_v1", b"batch_v1", "deletion_job", b"deletion_job", "job_type", b"job_type", "streaming", b"streaming", "streaming_aggregation", b"streaming_aggregation"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["job_type", b"job_type"]) -> typing_extensions.Literal["streaming", "batch_v1", "streaming_aggregation", "deletion_job"] | None: ...

global___FeatureSetJobType = FeatureSetJobType

class Streaming(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ONLINE_FIELD_NUMBER: builtins.int
    OFFLINE_FIELD_NUMBER: builtins.int
    @property
    def online(self) -> global___Online: ...
    @property
    def offline(self) -> global___Offline: ...
    def __init__(
        self,
        *,
        online: global___Online | None = ...,
        offline: global___Offline | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["offline", b"offline", "online", b"online", "sub_type", b"sub_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["offline", b"offline", "online", b"online", "sub_type", b"sub_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["sub_type", b"sub_type"]) -> typing_extensions.Literal["online", "offline"] | None: ...

global___Streaming = Streaming

class Online(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Online = Online

class Offline(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Offline = Offline

class BatchV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___BatchV1 = BatchV1

class StreamingAggregation(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROW_LEVEL_FIELD_NUMBER: builtins.int
    BACKFILL_FIELD_NUMBER: builtins.int
    COMPACTION_FIELD_NUMBER: builtins.int
    @property
    def row_level(self) -> global___RowLevel: ...
    @property
    def backfill(self) -> global___BackfillSubType: ...
    @property
    def compaction(self) -> global___Compaction: ...
    def __init__(
        self,
        *,
        row_level: global___RowLevel | None = ...,
        backfill: global___BackfillSubType | None = ...,
        compaction: global___Compaction | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "compaction", b"compaction", "row_level", b"row_level", "sub_type", b"sub_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["backfill", b"backfill", "compaction", b"compaction", "row_level", b"row_level", "sub_type", b"sub_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["sub_type", b"sub_type"]) -> typing_extensions.Literal["row_level", "backfill", "compaction"] | None: ...

global___StreamingAggregation = StreamingAggregation

class RowLevel(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RowLevel = RowLevel

class Compaction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Compaction = Compaction

class BackfillSubType(google.protobuf.message.Message):
    """Backfill already in used"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___BackfillSubType = BackfillSubType

class DeletionJob(google.protobuf.message.Message):
    """Deletion job"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BATCH_V1_DELETION_FIELD_NUMBER: builtins.int
    STREAMING_V1_DELETION_FIELD_NUMBER: builtins.int
    STREAMING_AGGREGATION_DELETION_FIELD_NUMBER: builtins.int
    @property
    def batch_v1_deletion(self) -> global___BatchV1DeletionJobSubType:
        """deletion of a BatchV1 featureset"""
    @property
    def streaming_v1_deletion(self) -> global___StreamingV1DeletionJobSubType:
        """deletion of a StreamingV1 featureset"""
    @property
    def streaming_aggregation_deletion(self) -> global___StreamingAggregationDeletionJobSubType:
        """deletion of a Streaming Aggregation Featureset"""
    def __init__(
        self,
        *,
        batch_v1_deletion: global___BatchV1DeletionJobSubType | None = ...,
        streaming_v1_deletion: global___StreamingV1DeletionJobSubType | None = ...,
        streaming_aggregation_deletion: global___StreamingAggregationDeletionJobSubType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_v1_deletion", b"batch_v1_deletion", "job_sub_type", b"job_sub_type", "streaming_aggregation_deletion", b"streaming_aggregation_deletion", "streaming_v1_deletion", b"streaming_v1_deletion"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["batch_v1_deletion", b"batch_v1_deletion", "job_sub_type", b"job_sub_type", "streaming_aggregation_deletion", b"streaming_aggregation_deletion", "streaming_v1_deletion", b"streaming_v1_deletion"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["job_sub_type", b"job_sub_type"]) -> typing_extensions.Literal["batch_v1_deletion", "streaming_v1_deletion", "streaming_aggregation_deletion"] | None: ...

global___DeletionJob = DeletionJob

class BatchV1DeletionJobSubType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___BatchV1DeletionJobSubType = BatchV1DeletionJobSubType

class StreamingV1DeletionJobSubType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StreamingV1DeletionJobSubType = StreamingV1DeletionJobSubType

class StreamingAggregationDeletionJobSubType(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StreamingAggregationDeletionJobSubType = StreamingAggregationDeletionJobSubType

class ExecutorStates(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTOR_STATES_FIELD_NUMBER: builtins.int
    @property
    def executor_states(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ExecutorState]: ...
    def __init__(
        self,
        *,
        executor_states: collections.abc.Iterable[global___ExecutorState] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["executor_states", b"executor_states"]) -> None: ...

global___ExecutorStates = ExecutorStates

class ExecutorState(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTOR_NAME_FIELD_NUMBER: builtins.int
    STATE_FIELD_NUMBER: builtins.int
    executor_name: builtins.str
    """Executor pod name"""
    state: global___FeatureSetLastExecutionState.ValueType
    """state of the running executor"""
    def __init__(
        self,
        *,
        executor_name: builtins.str = ...,
        state: global___FeatureSetLastExecutionState.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["executor_name", b"executor_name", "state", b"state"]) -> None: ...

global___ExecutorState = ExecutorState



import json
import typing as t

from qwak.exceptions import QwakException

from .base_input_adapter import BaseInputAdapter

try:
    import numpy as np
except ImportError:
    pass
    # Numpy is supported either by supplying it as a dependency (or sub-dependency)
    # in your Python project, or during the runtime


def _is_matched_shape(
    left: t.Optional[t.Tuple[int, ...]],
    right: t.Optional[t.Tuple[int, ...]],
) -> bool:  # pragma: no cover
    if (left is None) or (right is None):
        return False

    if len(left) != len(right):
        return False

    for i, j in zip(left, right):
        if i == -1 or j == -1:
            continue
        if i == j:
            continue
        return False
    return True


class NumpyInputAdapter(BaseInputAdapter):
    def __init__(
        self,
        dtype: t.Optional[t.Union[str]] = None,
        enforce_dtype: bool = False,
        shape: t.Optional[t.Tuple[int, ...]] = None,
        enforce_shape: bool = False,
    ):
        if dtype is not None and not isinstance(dtype, np.dtype):
            # Convert from primitive type or type string, e.g.:
            # np.dtype(float)
            # np.dtype("float64")
            try:
                dtype = np.dtype(dtype)
            except TypeError as e:
                raise QwakException(f'NumpyNdarray: Invalid dtype "{dtype}": {e}')

        self._dtype = dtype
        self._shape = shape
        self._enforce_dtype = enforce_dtype
        self._enforce_shape = enforce_shape

    def _verify_ndarray(
        self,
        obj,
        exception_cls: t.Type[Exception] = QwakException,
    ):
        if self._dtype is not None and self._dtype != obj.dtype:
            # ‘same_kind’ means only safe casts or casts within a kind, like float64
            # to float32, are allowed.
            if np.can_cast(obj.dtype, self._dtype, casting="same_kind"):
                obj = obj.astype(self._dtype, casting="same_kind")  # type: ignore
            else:
                msg = (
                    f'{self.__class__.__name__}: Expecting ndarray of dtype "{self._dtype}", but "{obj.dtype}" was '
                    f"received. "
                )
                if self._enforce_dtype:
                    raise exception_cls(msg)
                else:
                    print(msg)

        if self._shape is not None and not _is_matched_shape(self._shape, obj.shape):
            msg = (
                f'{self.__class__.__name__}: Expecting ndarray of shape "{self._shape}", but "{obj.shape}" was '
                f"received. "
            )
            if self._enforce_shape:
                raise exception_cls(msg)
            try:
                obj = obj.reshape(self._shape)
            except ValueError as e:
                print(f"{msg} Failed to reshape: {e}.")

        return obj

    def extract_user_func_arg(self, data: bytes):
        import numpy as np

        data = json.loads(data)
        try:
            res = np.array(data, dtype=self._dtype)  # type: ignore[arg-type]
        except ValueError:
            res = np.array(data)  # type: ignore[arg-type]
        res = self._verify_ndarray(res, QwakException)
        return [res]
